from qgis.core import *
from qgis.gui import *


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def ukraine_timemap_display(project_filename, id, date, description, feature, parent):
    """
    produce display text for a ukraine_timemap mapid.

    ukraine_timemap_display(
        "id",
        "description",
        "date",
    )
    """
    version = "1.11.1"

    return "<hr/>".join(
        [
            "{}: {}".format(
                date.toString("yyyy-MM-dd"),
                description,
            ),
            '{} / <a href="https://ukraine.bellingcat.com/?id={}">#{}</a>'.format(
                project_filename.split(".")[0],
                id,
                id,
            ),
            f"🇺🇦 ukraine timemap template {version}",
        ]
    )



import os

from blue_options import MARQUEE as default_MARQUEE
from blue_options.help.functions import get_help
from blue_objects import file, README
from blue_objects.env import ABCLI_PUBLIC_PREFIX

from blue_geo.catalog.README import build as build_catalog
from blue_geo.watch.README import items as watch_items, macros as watch_macros
from blue_geo.watch.targets.README import build as build_targets
from blue_geo.help.functions import help_functions
from blue_geo import NAME, VERSION, ICON, REPO_NAME


features = {
    "copernicus": {
        "description": "catalog: [Copernicus Data Space Ecosystem - Europe's eyes on Earth](https://dataspace.copernicus.eu/)",
        "icon": "🧊",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/copernicus.jpg?raw=true",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog/copernicus",
    },
    "SkyFox": {
        "description": "catalog: [Earth Data Store](https://earthdaily.github.io/EDA-Documentation/).",
        "icon": ICON,
        "thumbnail": "https://earthdaily.github.io/EDA-Documentation/Images/EarthDailyEDS.png",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog/SkyFox",
    },
    "EarthSearch": {
        "description": "catalog: [Earth Search by Element 84 (earth-search-aws)](https://stacindex.org/catalogs/earth-search#/).",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/viewer-aws-element84-com.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/catalog/EarthSearch",
    },
    "firms-area": {
        "description": "catalog: Fire Information for Resource Management System ([FIRMS](https://firms.modaps.eosdis.nasa.gov)).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/datacube-firms_area.jpg",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog/firms",
    },
    "ukraine-timemap": {
        "description": "catalog: [Bellingcat](https://www.bellingcat.com/) [Civilian Harm in Ukraine TimeMap](https://github.com/bellingcat/ukraine-timemap) dataset, available through [this UI](https://ukraine.bellingcat.com/) and [this API](https://bellingcat-embeds.ams3.cdn.digitaloceanspaces.com/production/ukr/timemap/api.json).",
        "icon": "🇺🇦",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/nbs/ukraine-timemap/QGIS.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/catalog/ukraine_timemap",
    },
    "vancouver-watching": {
        "description": f"catalog: Vancouver watching with AI, last build: [🔗]({ABCLI_PUBLIC_PREFIX}/test_vancouver_watching_ingest/animation.gif).",
        "icon": "🌈",
        "thumbnail": f"{ABCLI_PUBLIC_PREFIX}/2024-01-06-20-39-46-73614/2024-01-06-20-39-46-73614-2X.gif?raw=true",
        "url": "https://github.com/kamangir/Vancouver-Watching",
    },
    "geo-watch": {
        "description": "watch the planet's story unfold.",
        "icon": ICON,
        "thumbnail": f"{ABCLI_PUBLIC_PREFIX}/geo-watch-2024-09-06-Jasper-a/geo-watch-2024-09-06-Jasper-a-2X.gif",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/watch",
    },
    "catalog": {
        "description": "generalized STAC Catalogs.",
        "icon": ICON,
        "thumbnail": default_MARQUEE,
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog",
    },
    "datacube": {
        "description": "generalized STAC Items.",
        "icon": "🧊",
        "thumbnail": default_MARQUEE,
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/datacube",
    },
    "QGIS": {
        "description": "an AI terraform for [QGIS](https://www.qgis.org/).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/QGIS.jpg",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/QGIS/README.md",
    },
    "template": {
        "description": "",
        "icon": "",
        "thumbnail": default_MARQUEE,
        "url": "",
    },
}


items = [
    "{}[`{}`]({}) [![image]({})]({}) {}".format(
        details["icon"],
        feature,
        details["url"],
        details["thumbnail"],
        details["url"],
        details["description"],
    )
    for feature, details in features.items()
    if feature != "template"
]


def build() -> bool:
    return (
        all(
            README.build(
                items=items,
                cols=cols,
                path=os.path.join(file.path(__file__), suffix),
                macros=macros,
                ICON=ICON,
                NAME=NAME,
                VERSION=VERSION,
                REPO_NAME=REPO_NAME,
                help_function=lambda tokens: get_help(
                    tokens,
                    help_functions,
                    mono=True,
                ),
            )
            for suffix, items, cols, macros, in [
                ("..", items, 3, {}),
                ("catalog", [], -1, {}),
                ("datacube", [], -1, {}),
                ("watch", watch_items, -1, watch_macros),
                ("QGIS", [], -1, {}),
            ]
        )
        and build_catalog()
        and build_targets()
    )



if not QGIS_is_live:
    from log import log


class BLUE_GEO_QGIS_APPLICATION(object):
    def __init__(self, name, icon):
        self.name = name
        self.icon = icon

        log(self.name, "", icon=self.icon)

    def help(self):
        pass

    def log(self, message, note=""):
        log(message, note, icon=self.icon)



import os
import yaml
import shutil

if not QGIS_is_live:
    from log import log_error, log


def copy_file(
    source_filename: str,
    destination_filename: str,
) -> bool:
    try:
        # https://stackoverflow.com/a/8858026
        # better choice: copy2
        shutil.copyfile(source_filename, destination_filename)
    except:
        log_error(
            "copy_file({},{}): failed.".format(
                source_filename,
                destination_filename,
            )
        )
        return False

    log(
        "{} -> {}".format(
            source_filename,
            destination_filename,
        )
    )
    return True


def file_extension(filename: str) -> str:
    return os.path.splitext(filename)[1][1:]


def file_name(filename: str) -> str:
    _, filename = os.path.split(filename)

    return filename if "." not in filename else ".".join(filename.split(".")[:-1])


def file_name_and_extension(filename: str) -> str:
    return os.path.basename(filename)


def file_path(filename: str) -> str:
    return os.path.split(filename)[0]


def load_yaml(filename):
    with open(filename, "r") as file:
        return yaml.safe_load(file)


def file_with_extension(filename: str, extension: str) -> str:
    filename, _ = os.path.splitext(filename)

    return f"{filename}.{extension}"



from typing import Type, Union, List
from blue_geo.catalog.classes import list_of_catalog_classes, list_of_datacube_classes
from blue_geo.catalog.generic import (
    GenericCatalog,
    VoidCatalog,
    GenericDatacube,
    VoidDatacube,
)


def get_catalog_class(catalog_name: str) -> Type[GenericCatalog]:
    for catalog_class in list_of_catalog_classes:
        if catalog_class.name == catalog_name:
            return catalog_class

    return VoidCatalog


def get_catalog(catalog_name: str) -> GenericCatalog:
    return get_catalog_class(catalog_name)()


def get_datacube(datacube_id: str) -> GenericDatacube:
    return get_datacube_class(datacube_id)(datacube_id)


def get_datacube_class(datacube_id: str) -> Type[GenericDatacube]:
    for datacube_class in list_of_datacube_classes:
        success, _ = datacube_class.parse_datacube_id(datacube_id)
        if success:
            return datacube_class

    return VoidDatacube


def get_datacube_class_in_catalog(
    catalog_name: str,
    collection_name: str,
) -> Type[GenericDatacube]:
    catalog_class = get_catalog_class(catalog_name)

    return (
        [
            datacube_class
            for datacube_class in get_list_of_datacube_classes(catalog_class)
            if datacube_class.name == collection_name
        ]
        + [GenericDatacube]
    )[0]


def get_list_of_collections(
    catalog_class: Union[Type[GenericCatalog], str],
) -> List[str]:
    catalog = get_catalog(catalog_class)

    return catalog.get_list_of_collections()


def get_list_of_datacube_classes(
    catalog_class: Union[Type[GenericCatalog], str],
) -> List[Type[GenericDatacube]]:
    if isinstance(catalog_class, str):
        catalog_class = get_catalog_class(catalog_class)

    return [
        datacube_class
        for datacube_class in list_of_datacube_classes
        if datacube_class.catalog.__class__ == catalog_class
    ]



import os
import glob

if not QGIS_is_live:
    from application import BLUE_GEO_QGIS_APPLICATION
    from log import log
    from project import project
    from QGIS import QGIS


class BLUE_GEO_QGIS_APPLICATION_VANWATCH(BLUE_GEO_QGIS_APPLICATION):
    def __init__(self):
        super().__init__("vanwatch", "🌈")

    def help(self):
        self.log(
            "vanwatch.list_layers()",
            "list vanwatch layers.",
        )
        self.log(
            "vanwatch.load(count=<count>, timed=True, offset=<offset>, prefix=<prefix>)",
            "load layers.",
        )
        self.log(
            "vanwatch.unload(prefix)",
            "unload prefix*.",
        )

    def list_layers(self):
        return sorted(
            [
                os.path.splitext(os.path.basename(filename))[0]
                for filename in glob.glob(
                    os.path.join(
                        project.path,
                        "*.geojson",
                    )
                )
            ]
        )

    def load(
        self,
        prefix="",
        count=-1,
        offset=0,
        refresh=True,
        timed=False,
    ) -> bool:
        list_layers = self.list_layers()

        list_layers = list(reversed(list_layers))
        if offset:
            list_layers = list_layers[offset:]
        if count != -1:
            list_layers = list_layers[:count]

        for layer_name in list_layers:
            if not layer_name.startswith(prefix):
                continue

            filename = os.path.join(project.path, f"{layer_name}.geojson")

            QGIS.load(
                filename,
                layer_name,
                "template-timed" if timed else "template",
                refresh=False,
            )

        self.log(f"loaded {len(list_layers)} layer(s).")

        if refresh:
            QGIS.refresh()

    def unload(self, prefix="", refresh=True):
        log(prefix, icon="🗑️")

        for layer_name in [
            layer_name
            for layer_name in QGIS.list_of_layers()
            if layer_name.startswith(prefix)
        ]:
            QGIS.unload(layer_name, refresh=False)

        if refresh:
            QGIS.refresh()


vanwatch = BLUE_GEO_QGIS_APPLICATION_VANWATCH()
QGIS.add_application(vanwatch)



from typing import Tuple, Dict, Any, List
from datetime import datetime, timedelta
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point

from blue_objects import file, objects
from blue_objects import metadata

from blue_geo.catalog.firms import FirmsCatalog
from blue_geo.catalog.generic import GenericDatacube
from blue_geo.catalog.firms.area.enums import Area, Source
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo import env
from blue_geo.logger import logger


class FirmsAreaDatacube(GenericDatacube):
    name = "area"

    catalog = FirmsCatalog()

    query_args = {
        "area": {
            "default": Area.default().name,
            "help": "|".join(Area.values()),
        },
        "date": {
            "default": (datetime.now() - timedelta(days=5)).strftime("%Y-%m-%d"),
            "help": "<yyyy-mm-dd>",
        },
        "depth": {
            "type": int,
            "default": 1,
            "help": "1..10",
        },
        "source": {
            "default": Source.default().name,
            "help": "|".join(Source.values()),
        },
    }

    QGIS_template = env.BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA

    def __init__(
        self,
        datacube_id: str = "",
        source: Source = Source.default(),
        area: Area = Area.default(),
        date: str = "",
        depth: int = 1,
    ):
        super().__init__(datacube_id)

        self.url_prefix = "https://firms.modaps.eosdis.nasa.gov/api/area"
        self.map_key = env.FIRMS_MAP_KEY

        if datacube_id:
            success, args = FirmsAreaDatacube.parse_datacube_id(datacube_id)
            assert success

            area = args["area"]
            source = args["source"]
            depth = args["depth"]
            date = args["date"]

        self.area: Area = area
        self.source: Source = source

        self.depth: int = depth

        self.date: str = (
            date if date else (datetime.now() - timedelta(days=5)).strftime("%Y-%m-%d")
        )

        self.build_datacube_id()

    def build_datacube_id(self):
        self.datacube_id = "datacube-{}-{}-{}-{}-{}-{}".format(
            self.catalog.name,
            self.name,
            self.area.name.lower(),
            self.source.name,
            self.date,
            self.depth,
        )

    @property
    def description(self) -> str:
        return "{}, area:{}, source:{} ({})".format(
            super().description,
            self.area.name.lower(),
            self.source.name,
            self.source.description,
        )

    def ingest(
        self,
        dryrun: bool = False,
        overwrite: bool = False,
        scope: str = "metadata",
    ) -> Tuple[
        bool,
        gpd.GeoDataFrame,
    ]:
        success, _ = super().ingest(dryrun, overwrite, scope)
        if not success:
            return success, gpd.GeoDataFrame()

        csv_filename = objects.path_of("firms_area.csv", self.datacube_id, create=True)
        if not file.download(
            self.ingest_url(),
            csv_filename,
        ):
            return False, gpd.GeoDataFrame()

        data = pd.read_csv(csv_filename)
        if data.empty:
            return False, gpd.GeoDataFrame()

        logger.info(f"loaded {len(data):,} point(s).")

        gdf = gpd.GeoDataFrame(
            data,
            geometry=[
                Point(xy)
                for xy in zip(
                    data.longitude,
                    data.latitude,
                )
            ],
            crs="EPSG:4326",  # WGS84
        )

        if not file.save_geojson(
            objects.path_of("firms_area.geojson", self.datacube_id),
            gdf,
        ):
            return False, gdf

        if not metadata.post_to_object(
            self.datacube_id,
            "datacube",
            {
                "id": self.datacube_id,
                "source": self.source.name,
                "area": self.area.name,
                "date": self.date,
                "depth": self.depth,
                "len": len(gdf),
            },
        ):
            return False, gdf

        return True, gdf

    def ingest_url(self, html: bool = False) -> str:
        return "{}/{}/{}/{}/{}/{}/{}".format(
            self.url_prefix,
            "html" if html else "csv",
            self.map_key,
            self.source.name,
            self.area.name.lower(),
            self.depth,  # day_range
            self.date,
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        return [
            f"firms_area.{extension}"
            for extension in [
                "geojson",
                "csv",
            ]
        ]

    @classmethod
    def parse_datacube_id(cls, datacube_id: str) -> Tuple[
        bool,
        Dict[str, Any],
    ]:
        success, _ = super().parse_datacube_id(datacube_id)
        if not success:
            return False, {}

        # datacube-firm-area-<area>-<source>-yyyy-mm-dd-depth
        # datacube-firms-area-world-MODIS_NRT-2024-07-20-1
        segments = datacube_id.split("-")
        if len(segments) < 9:
            return False, {}

        if segments[2] != cls.name:
            return False, {}

        area_str = segments[3]
        if area_str not in Area.values():
            return False, {}

        source_str = segments[4]
        if source_str not in Source.values():
            return False, {}

        date = "{}-{}-{}".format(segments[5], segments[6], segments[7])

        depth_str = segments[8]
        if not depth_str.isdigit():
            return False, {}

        return (
            True,
            {
                "area": Area[area_str.upper()],
                "source": Source[source_str],
                "date": date,
                "depth": int(depth_str),
            },
        )

    @classmethod
    def query(cls, object_name: str, **kwargs) -> bool:
        logger.info(f"🔎 {cls.__name__}.query -> {object_name}")

        datacube = cls(**kwargs)

        logger.info(f"🧊 {datacube.description}")

        return metadata.post_to_object(
            object_name,
            "datacube_id",
            [datacube.datacube_id],
        )



import os

if not QGIS_is_live:
    from log import verbose, log
    from fileio import load_yaml


class ABCLI_QGIS_Project(object):
    def help(self):
        pass

    @property
    def filename(self):
        return QgsProject.instance().fileName()

    @property
    def list_of_layers(self):
        output = [layer.name() for layer in QgsProject.instance().mapLayers().values()]
        if verbose:
            log(
                f"{len(output)} layer(s)",
                ", ".join(output) if verbose else "",
                icon="🔎",
            )
        return output

    @property
    def metadata(self):
        filename = os.path.join(self.path, "metadata.yaml")

        if not os.path.exists(filename):
            return {"error": f"{filename}: file not found."}

        return load_yaml(filename)

    @property
    def name(self):
        return QgsProject.instance().homePath().split(os.sep)[-1]

    @property
    def path(self):
        return QgsProject.instance().homePath()

    def trim_empty_groups(self):
        root = QgsProject.instance().layerTreeRoot()
        groups_to_check = [root]

        while groups_to_check:
            group = groups_to_check.pop(0)
            children = group.children()
            empty = all(
                child.nodeType() == 0 and child.children() == [] for child in children
            )

            if empty and group != root:
                parent = group.parent()
                parent.removeChildNode(group)
            else:
                groups_to_check.extend(
                    [child for child in children if child.nodeType() == 0]
                )


project = ABCLI_QGIS_Project()



from typing import List, Dict
import os

from blue_options import string
from blue_objects import file
from blue_geo.watch.targets.target_list import TargetList
from blue_objects.env import ABCLI_PUBLIC_PREFIX

from blue_geo import REPO_NAME


list_of_targets = {
    "chilcotin-river-landslide": {
        "objects": {
            #            "test_blue_geo_watch_v4-diff-chilcotin-river-landslide-test": [
            #                f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
            #            ],
            "test_blue_geo_watch_v4-modality-chilcotin-river-landslide-test": [
                f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
            ],
            "geo-watch-2024-08-31-chilcotin-c": [
                "L1C and L2A mixed",
                "`2024-07-30/2024-08-09`",
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-199-11f9b5497ef0)",
            ],
            "geo-watch-2024-09-01-chilcotin-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-201-d64e9bb3716b)",
            ],
            "geo-watch-2024-09-01-chilcotin-c": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-202-d59ba811398b)",
                "[on reddit](https://www.reddit.com/r/bash/comments/1f9cvyx/a_bash_python_tool_to_watch_a_target_in_satellite/).",
            ],
            "geo-watch-Chilcotin-2024-11-03": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-307-b4ed600efc16)",
            ],
        },
        "thumbnail": {
            "scale": 4,
        },
    },
    "burning-man-2024": {
        "objects": {
            "geo-watch-2024-09-04-burning-man-2024-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-205-c272a95ce266)",
            ]
        },
    },
    "Mount-Etna": {
        "objects": {
            "geo-watch-2024-09-04-Mount-Etna-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-205-c272a95ce266)",
            ]
        },
    },
    "Fagradalsfjall": {
        "objects": {
            "geo-watch-2024-09-04-Fagradalsfjall-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-206-f7996520dc15)",
            ]
        },
    },
    "Jasper": {
        "objects": {
            "geo-watch-2024-09-06-Jasper-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-208-7063fca1423b)",
            ],
            "geo-watch-Jasper-2024-11-03": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-307-b4ed600efc16)",
            ],
        },
    },
    "Leonardo": {
        "objects": {
            "test_blue_geo_watch_v4-diff-Leonardo-test": [
                f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
            ],
            "test_blue_geo_watch_v4-modality-Leonardo-test": [
                f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
            ],
            "geo-watch-2024-09-30-Leonardo-g": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-237-99db71023445)",
            ],
            "geo-watch-Leonardo-2024-10-05-a": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-249-44ba1dcd2321)",
            ],
            "geo-watch-Leonardo-2024-10-06-a": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-255-1f2a8f1ccef5)",
            ],
            "geo-watch-2024-10-27-16-17-36-12059": [
                "[dev note](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-294-1bba1bdc3c16)",
            ],
        },
        "thumbnail": {
            "scale": 4,
        },
    },
    "bellingcat-2024-09-27-nagorno-karabakh": {
        "objects": {
            "bellingcat-2024-09-27-nagorno-karabakh-2024-10-01-c-b": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-241-3e25857747a5)",
            ],
            "bellingcat-2024-09-27-nagorno-karabakh-b": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-244-a5f9b7959748)",
            ],
            "bellingcat-2024-09-27-nagorno-karabakh-6X-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-245-18f6d15e5fbd)",
            ],
            "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-05-b": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-250-847d3d5f0d6e)",
            ],
            "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-06-a": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-255-1f2a8f1ccef5)",
            ],
        },
        "thumbnail": {
            "scale": 4,
        },
    },
    "elkhema": {
        "objects": {
            "geo-watch-elkhema-2024-2024-10-05-a-b": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-251-a68fab7f52b8)",
            ]
        },
        "thumbnail": {
            "scale": 4,
        },
        "title": "elkhema ⛺️",
    },
    "Cache-Creek": {
        "objects": {
            "geo-watch-Cache-Creek-2024-10-06-a": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-253-8f12ef5bd8fc)",
            ],
            "geo-watch-Cache-Creek-2x-wider-2024-10-06-a": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-256-c1b564c9f89e)"
            ],
            "geo-watch-Cache-Creek-2024-11-05": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-308-2d1de0179db1)",
            ],
            "geo-watch-Cache-Creek-2x-wider-2024-11-05": [
                "[dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-309-ee21dd00730c)",
            ],
        },
        "thumbnail": {
            "scale": 4,
        },
    },
    "Silver-Peak": {
        "objects": {
            "geo-watch-Silver-Peak-2024-10-12-a": [
                "[dev notes](https://medium.com/@arash-kamangir/%EF%B8%8F-conversations-with-ai-267-8720fd3460d0)",
            ],
        },
        "thumbnail": {
            "scale": 4,
        },
    },
}

targets_path = file.path(__file__)

items: List[str] = []
for target_name in sorted(list_of_targets.keys()):
    target_info = list_of_targets[target_name]

    list_of_objects = target_info["objects"]

    target_README = f"./targets/md/{target_name}.md"

    target_title = "`{}`".format(target_info.get("title", target_name))

    items += [
        (
            f"## [{target_title}]({target_README})"
            if file.exists(os.path.join(targets_path, target_README))
            else f"## {target_title}"
        ),
    ]

    target_list = TargetList()
    items += target_list.get(target_name).urls_as_str()

    if list_of_objects:
        thumbnail_info = target_info.get("thumbnail", {})

        thumbnail_index = thumbnail_info.get("index", -1)

        thumbnail_scale = thumbnail_info.get("scale", 2)
        thumbnail_scale_str = f"-{thumbnail_scale}X" if thumbnail_scale != 1 else ""

        thumbnail_object_name = list(list_of_objects.keys())[thumbnail_index]
        items += [
            "",
            f"![image]({ABCLI_PUBLIC_PREFIX}/{thumbnail_object_name}/{thumbnail_object_name}{thumbnail_scale_str}.gif?raw=true&random={string.random()})",
        ]

    items += [
        "- {}.".format(
            ", ".join(
                [
                    f"[`{object_name}`]({ABCLI_PUBLIC_PREFIX}/{object_name}.tar.gz)",
                    f"[gif]({ABCLI_PUBLIC_PREFIX}/{object_name}/{object_name}.gif)",
                ]
                + description
            )
        )
        for object_name, description in list_of_objects.items()
    ]

    items += [""]

object_name = "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-05-b"
macros: Dict[str, str] = {
    "--scale-note--": [
        "ℹ️ suffix published gif urls with `-2X` and `-4X` for different scales. example: {}.".format(
            ", ".join(
                [
                    "[{}X]({}/{}/{}{}.gif)".format(
                        scale,
                        ABCLI_PUBLIC_PREFIX,
                        object_name,
                        object_name,
                        "" if scale == 1 else f"-{scale}X",
                    )
                    for scale in [1, 2, 4]
                ]
            )
        )
    ]
}



from typing import List, Dict

from blue_objects import host, file, host

from blue_geo.catalog.SkyFox.classes import SkyFoxCatalog
from blue_geo.catalog.generic.generic.stac import STACDatacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo import env
from blue_geo.logger import logger

rgb_suffixes = [f"SRE_B{band_index}.tif" for band_index in [7, 4, 3]]


class SkyFoxVenusDatacube(STACDatacube):
    catalog = SkyFoxCatalog()

    collection = "venus-l2a"

    name = "Venus"

    QGIS_template = env.BLUE_GEO_QGIS_TEMPLATE_DATACUBE_SKYFOX_VENUS

    def generate(
        self,
        modality: str,
        overwrite: bool = False,
    ) -> str:
        # TODO: clean-up in the next refactor 🤦🏽
        if modality != "rgb":
            logger.error(f"{modality}: modality is not implemented.")
            return ""

        list_of_colors = ["red", "green", "blue"]

        filenames: Dict[str, str] = {}
        for suffix, color in zip(rgb_suffixes, list_of_colors):
            candidates = self.list_of_files(DatacubeScope(suffix))
            if not candidates:
                logger.error(f"cannot find {suffix}.")
                return ""

            filenames[color] = self.full_filename(candidates[0])

        rgb_filename = filenames["red"].replace(rgb_suffixes[0], "SRE_RGB.tif")
        if file.exists(rgb_filename) and not overwrite:
            logger.info(f"✅ {rgb_filename}")
            return ""

        return " ".join(
            [
                "gdal_merge.py",
                "-separate",
                f"-o {rgb_filename}",
            ]
            + [filenames[color] for color in list_of_colors]
        )

    def ingest_filename(
        self,
        filename: str,
        overwrite: bool = False,
        verbose: bool = True,
    ) -> bool:
        if super().ingest_filename(filename, overwrite, verbose):
            return True

        s3_uri = ""
        for item in self.metadata["Item"].assets.values():
            if item.href.endswith(filename):
                s3_uri = item.href
                break

        if not s3_uri:
            logger.info(f"{filename}: file not found.")
            return False

        # https://registry.opendata.aws/venus-l2a-cogs/
        return host.shell(
            "aws s3 cp --no-sign-request {} {}".format(
                s3_uri,
                self.full_filename(filename),
            ),
            log=verbose,
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        raw_datacube_id = self.raw_datacube_id()

        output = scope.filter(
            [
                {
                    "filename": (value.href.split(f"{raw_datacube_id}/", 1)[1]),
                }
                for value in self.metadata["Item"].assets.values()
                if raw_datacube_id in value.href
            ],
            needed_for_rgb=lambda filename: any(
                filename.endswith(suffix) for suffix in rgb_suffixes
            ),
            verbose=verbose,
        )

        if scope.rgb:
            suffix = rgb_suffixes[0]
            candidates = self.list_of_files(DatacubeScope(suffix))
            if candidates:
                rgb_filename = candidates[0].replace(suffix, "SRE_RGB.tif")
                output += [rgb_filename]

        return output
